package org.bdware.sc.handler;

import io.netty.channel.ChannelHandlerContext;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.doip.codec.doipMessage.DoipMessage;
import org.bdware.doip.codec.operations.BasicOperations;
import org.bdware.doip.endpoint.server.DoipRequestHandler;
import org.bdware.doip.endpoint.server.NettyServerHandler;
import org.bdware.doip.endpoint.server.Op;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

public class DOOPRequestHandler implements DoipRequestHandler {
    Map<String, Method> handlers;
    static Logger logger = LogManager.getLogger(NettyServerHandler.class);
    protected DOOPRepositoryHandler doopRepoHandler;

    public DOOPRequestHandler(DOOPRepositoryHandler doopRepoHandler) {
        handlers = new HashMap<>();
        this.doopRepoHandler = doopRepoHandler;
        Class handlerClass = doopRepoHandler.getClass();

        // 建立doopRepoHandler这个类中，方法名和对应的Method的对应关系
        while (handlerClass != Object.class) {
            putDoipHandlerMethod(handlerClass);
            Class[] interfaces = handlerClass.getInterfaces();
            for (Class clz : interfaces) {
                putDoipHandlerMethod(clz);
            }
            handlerClass = handlerClass.getSuperclass();
        }
    }

    private void putDoipHandlerMethod(Class handlerClass) {
        Method[] methods = handlerClass.getDeclaredMethods();
        for (Method m : methods) {
            Op a = m.getAnnotation(Op.class);
            if (a != null) {
//                logger.debug("method annotation: " + a.op().getName());
//                logger.debug("method annotation: " + a.name());
                if (a.op() != BasicOperations.Extension) {
                    putHandler(a.op().getName(), m);
                } else {
                    putHandler(a.name(), m);
                }
            }
        }
    }

    private void putHandler(String name, Method m) {
        if (handlers.containsKey(name)) {
//            logger.error("Duplicated operation handler:" + name + " methodName:" + m.getName());
            return;
        }
        logger.debug("[Register operation] name: " + name);
        m.setAccessible(true);
        handlers.put(name, m);
    }

    @Override
    public DoipMessage onRequest(ChannelHandlerContext ctx, DoipMessage msg) {
        String str = msg.header.parameters.operation;
        logger.debug("[Call operation] name: " + str);
        if (str != null) {
            Method m;
            m = handlers.get(str);
            if (m == null) m = handlers.get(BasicOperations.Unknown.getName());
            if (m != null) {
                try {
                    return (DoipMessage) m.invoke(doopRepoHandler, msg);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }
}
